<?php
/* this library uses GD to resize images, ImageMagick is also not bound by PHP's memory limit, GD might fail for images larger than 10MBytes */
require_once("./lib/misc.php");

// Link image type to correct image loader and saver
// - makes it easier to add additional types later on
// - makes the function easier to read
// creditz https://pqina.nl/blog/creating-thumbnails-with-php/
const IMAGE_HANDLERS = [
    IMAGETYPE_JPEG => [
        'load' => 'imagecreatefromjpeg',
        'save' => 'imagejpeg',
        'quality' => 100
    ],
    IMAGETYPE_PNG => [
        'load' => 'imagecreatefrompng',
        'save' => 'imagepng',
        'quality' => 0
    ],
    IMAGETYPE_GIF => [
        'load' => 'imagecreatefromgif',
        'save' => 'imagegif'
    ]
];

/* pass a $dir that contains images and it will automatically
 * 1) generate file.thumbnail.jpg from every file.jpg (if missing)
 * 2) echo out html for viewing the file.thumbnail.jpg
 * 
 * @param $dir  -   the directory in which file.jpg pictures are
 * @param $link_to_file -   if every file.thumbnail.jpg should be enclosed by a link to the original (full sized) file
 */
function GenerateAndShowThumbnails($dir,$link_to_file = true)
{
    $array_of_files = listFiles($dir);
    foreach ($array_of_files as $file)
    {
        if (strpos($file, "thumbnail") !== false)
        {
            continue; // ignore that file, it already is a thumbnail (to not create thumbnails of thumbnails)
        }

        $thumbnail_fullpath = GenerateNameThumbnail($file,$dir,"thumbnail_fullpath","thumbnail");
        $fullpath_original = GenerateNameThumbnail($file,$dir,"file_fullpath","thumbnail");
        
        // if the thumbnail does not exist, generate it
        if(!file_exists($thumbnail_fullpath))
        {
            ImageResize($fullpath_original, $thumbnail_fullpath, "100", null);
        }
        
        if($link_to_file)
        {
            echo '<a href="'.$fullpath_original.'" target="_blank">';
        }
        
        echo '                                <div class="subElement">
                                                <img src="'.$thumbnail_fullpath.'">
                                              </div>';
        
        if($link_to_file)
        {
            echo '</a>';
        }
    }
}

/**
 * @param $src - a valid file location
 * @param $dest - a valid file target
 * @param $targetWidth - desired output width
 * @param $targetHeight - desired output height or null
 */
function ImageResize($src, $dest, $targetWidth, $targetHeight = null)
{
    // 1. Load the image from the given $src
    // - see if the file actually exists
    // - check if it's of a valid image type
    // - load the image resource

    // get the type of the image
    // we need the type to determine the correct loader
    $type = exif_imagetype($src);

    // if no valid type or no handler found -> exit
    if (!$type || !IMAGE_HANDLERS[$type]) {
        return null;
    }

    // load the image with the correct loader
    $image = call_user_func(IMAGE_HANDLERS[$type]['load'], $src);

    // no image found at supplied location -> exit
    if (!$image) {
        return null;
    }

    // 2. Create a thumbnail and resize the loaded $image
    // - get the image dimensions
    // - define the output size appropriately
    // - create a thumbnail based on that size
    // - set alpha transparency for GIFs and PNGs
    // - draw the final thumbnail

    // get original image width and height
    $width = imagesx($image);
    $height = imagesy($image);

    // maintain aspect ratio when no height set
    if($targetHeight == null)
    {

        // get width to height ratio
        $ratio = $width / $height;

        // if is portrait
        // use ratio to scale height to fit in square
        if ($width > $height)
        {
            $targetHeight = floor($targetWidth / $ratio);
        }
        // if is landscape
        // use ratio to scale width to fit in square
        else
        {
            $targetHeight = $targetWidth;
            $targetWidth = floor($targetWidth * $ratio);
        }
    }

    // create duplicate image based on calculated target size
    $thumbnail = imagecreatetruecolor($targetWidth, $targetHeight);

    // set transparency options for GIFs and PNGs
    if($type == IMAGETYPE_GIF || $type == IMAGETYPE_PNG)
    {
        // make image transparent
        imagecolortransparent($thumbnail,imagecolorallocate($thumbnail, 0, 0, 0));

        // additional settings for PNGs
        if ($type == IMAGETYPE_PNG)
        {
            imagealphablending($thumbnail, false);
            imagesavealpha($thumbnail, true);
        }
    }

    // copy entire source image to duplicate image and resize
    imagecopyresampled(
        $thumbnail,
        $image,
        0, 0, 0, 0,
        $targetWidth, $targetHeight,
        $width, $height
    );


    // 3. Save the $thumbnail to disk
    // - call the correct save method
    // - set the correct quality level

    // save the duplicate version of the image to disk
    return call_user_func(IMAGE_HANDLERS[$type]['save'],$thumbnail,$dest,IMAGE_HANDLERS[$type]['quality']);
}
?>